# LipophilicityEfficiency.py - Lipophilicity efficiency plugin script for StarDrop
#
# Copyright (C) Optibrium Ltd 2010
# stardrop-support@optibrium.com
#
# This script allows the user to calculate a prediction of
# Lipophilicity Efficiency for a given column of target
# affinity (pKi) values

# For advice or feedback, contact stardrop-support@optibrium.com

import app
import math
import PythonQt
import DataSetUtils


def getLE(dataset):
    ds = app.Dataset()
    ds.parse(dataset)
    showcols = {}
    logpcols = {}
    idcol = -1
    logpcol = -1
    i = 0
    while i < ds.columnSz():
        type = ds.columnType(i)
        # look for logP columns
        name = ds.columnName(i)
        if name.upper().find('LOGP') > -1 or name.upper().find('LOG P') > -1:
            logpcols[ds.columnName(i)] = i
        # remove columns of the wrong type
        if type == app.cont:
            showcols[ds.columnName(i)] = i
        i += 1
    if len(logpcols) < 1:
        return 'No logP values available - please calculate and retry', False
    if len(showcols) < 1:
        return 'No data available - pKi values required', False
    else:
        logpres = True
        logpcol = list(logpcols.values())[0]
        if len(logpcols) > 1:
            logpdlg = PythonQt.ColumnPicker()
            logpdlg.setWindowTitle("Lipophilicity Efficiency")
            logpdlg.setLabel("Select column containing logP values:")
            logpdlg.setColumns(logpcols)
            logpres = logpdlg.run()
            if logpres == True:
                logpcolpicked = logpdlg.getColumn()
                logpcol = logpcols.get(logpcolpicked)
        if logpres == True:
            dlg = PythonQt.ColumnPicker()
            dlg.setWindowTitle("Lipophilicity Efficiency")
            dlg.setLabel("Select column containing pKi or micromolar Ki values:")
            dlg.setColumns(showcols)
            res = dlg.run()
            if res == True:
                col = dlg.getColumn()
                # find the pKi values
                colindex = showcols.get(col)
                if colindex >= 0:
                    return calculate(ds, colindex, logpcol)
                else:
                    return 'Inappropriate column selected', False


def calculate(ds, col, logpcol):
    # calculate Lipophilicity Efficiency for each row
    efficiencies = []
    units = ds.columns[col][6][1][0]
    for i in range(ds.rowSz()):
        temp = app.Dataset.createInvalidEntry(app.cont)
        try:
            if units == 5:  # pKi data
                pkival = ds.get(col, i)[1]
                pkierr = ds.get(col, i)[2]
            else:  # assume micromolar
                pkival = -math.log10((ds.get(col, i)[1]) / 1000000)
                pkierr = 0
            logpval = ds.get(logpcol, i)[1]
            logperr = ds.get(logpcol, i)[2]
            le = pkival - logpval
            if pkierr > 1000000000 or logperr > 1000000000:
                leerr = logperr
                if pkierr > logperr:
                    leerr = pkierr
            else:
                leerr = math.sqrt((pkierr * pkierr) + (logperr * logperr))
            t1 = temp[0]
            t2 = ("True", "", "")
            t3 = (0, le, leerr)
            entry = (t1, t2, t3)
            efficiencies.append(entry)
        except:
            efficiencies.append(temp)
    # add new column
    lecol = app.Dataset.meta('Lipophilicity Efficiency (' + ds.columnName(col) + ')', app.cont, ((0, 100), (0, 0)))
    ds.insertColumn(lecol, efficiencies, col + 1)
    return (ds.packColumn(col + 1), col + 1), True


def calculateLipophilicityEfficiency(stardrop_appui):
    dataset = stardrop_appui.importDS("")
    result = getLE(dataset)
    if result[1] == False:
        if len(result[0]) > 0:
            stardrop_appui.show(result[0])
        else:
            stardrop_appui.show('Calculation failed')
    else:
        stardrop_appui.startMacro("")
        (data, index) = result[0]
        stardrop_appui.addColumnToDataSet('', index, data)
        stardrop_appui.stopMacro("")


def get_stardrop_definitions():
    definitions = [
        {
            'script_name': 'Calculate Lipophilicity Efficiency',
            'callback': calculateLipophilicityEfficiency
        }
    ]

    return definitions
